module Page.Landing exposing
    ( Model
    , Msg
    , view
    , init
    , update
    , subscriptions
    )
import Element exposing (Element)
import Websockets
import Ports
import Json.Decode as Decode
import Json.Encode as Encode exposing (Value)
import Html.Attributes exposing (placeholder)
import Element.Input
import Element.Background

type alias Model = {
    time : String,
    greetWidgetText : String
    }
type DownMsgLanding
    = Greeting String
    | TimeUpdate String

decodeDownMsgLanding : Decode.Decoder DownMsgLanding
decodeDownMsgLanding =
    Decode.oneOf
        [ Decode.map Greeting (Decode.field "Greeting" Decode.string)
        , Decode.map TimeUpdate (Decode.field "TimeUpdate" Decode.string)
        ]

decodeDownMsg : Decode.Decoder Msg
decodeDownMsg =
    Decode.map DownMsg decodeDownMsgLanding

type UpMsgLanding = RequestGreet String

encodeUpMsgLanding : UpMsgLanding -> Value
encodeUpMsgLanding msg =
    case msg of
        RequestGreet name ->
            Encode.object
                [ ( "RequestGreet", Encode.string name ) ]

type Msg
    = DownMsg DownMsgLanding
    | UpMsg UpMsgLanding
    | DecodeError Decode.Error
    | GreetWidgetText String
    | NoOp

init : () -> Model
init flags = {
    time = "time not yet set",
    greetWidgetText = ""
    }

subscriptions : Model -> Sub Msg
subscriptions model =
    Ports.socketOnEvent
        (Websockets.EventHandlers
            (\_ -> NoOp)
            (\_ -> NoOp)
            (\_ -> NoOp)
            (\message ->
                case Decode.decodeString decodeDownMsg message.data of
                    Ok msg -> msg
                    Err err -> DecodeError err
            )
            (\_ -> NoOp)
        )

update : Msg -> Model -> (Model, Cmd Msg)
update msg model =
    case msg of
        DownMsg (TimeUpdate time) -> ( {model | time = time}, Cmd.none )
        GreetWidgetText text -> ( {model | greetWidgetText = text}, Cmd.none )
        _ -> (model, Cmd.none)

greetWidget : Model -> Element Msg
greetWidget model =
    let
        idleBlue = Element.Background.color     <| Element.rgb255 18 147 217
        focusBlue = Element.Background.color    <| Element.rgb255 18 125 184

        myButton = Element.Input.button
            [ idleBlue
            , Element.focused [ focusBlue ]
            ]
            { onPress = Just <| DownMsg <| Greeting model.greetWidgetText
            , label = Element.text "My Button"
            }
        placeholder  = case model.greetWidgetText of
            "" -> Just <| Element.Input.placeholder [] <| Element.text "Type Your Name"
            _ -> Just <| Element.Input.placeholder [] <| Element.text ""
        textInput =
            Element.Input.text []
                { onChange = GreetWidgetText
                , text = model.greetWidgetText
                , placeholder = placeholder
                , label = Element.Input.labelHidden "Greet"
                }
    in
        Element.row []
            [ textInput ]

view : Model -> Element Msg
view model =
    Element.column []
        [ Element.text "Landing"
        , Element.text <| "Current time is : " ++ model.time
        , greetWidget model
        ]
